package eLecture;



import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import org.jdom.CDATA;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;

import generator.Constants;
import generator.SvgDocument;
import generator.SvgElement;
import generator.SVGUtil;

/**
 * @author seb
 *
 * helper methods to construct SVG document
 * 
 */
public class GeneratorHelper {
	
	private Document labels;
	private String lang;
	
	public GeneratorHelper(String lang) throws Exception {
		this.lang = lang;
		File labelFile = new File("labels.xml");
		if (!labelFile.exists()) {
			System.err.println("Error: 'labels.xml' not found! " +
					"Program terminated!");
			throw new Exception();
		}
		SAXBuilder builder = 
            new SAXBuilder("org.apache.xerces.parsers.SAXParser", false);
		// TODO: validate against DTD labesl.dtd
        try {
			labels = builder.build(new FileInputStream(labelFile));
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("Error: can't proceed 'label.xml'! " +
					"Program terminated!");
			throw new Exception();
		} 
	}
	
	/**
	 * 
	 * create a group element "g" width 2 child elements "animate". One for reducing
	 * opacity of g to 0 and one for increasing opacity to 100%.
	 * 
	 * @param root father element on which new group is attached
	 * @return reference to group element
	 */
	public SvgElement createHidableGroup(SvgElement root) {
		SvgElement group = SVGUtil.createElement("g", root);
		// add hide/unhide functionality: mousedown hides and mouseup unhides
		SVGUtil.createElement("animate", group, new String[][] {{"fill", "freeze"},
			{"attributeName", "opacity"},{"begin", "hide_slide_button.mouseover"},
			{"to", "0"}, {"dur", "1s"}});
		SVGUtil.createElement("animate", group, new String[][] {{"fill", "freeze"},
			{"attributeName", "opacity"},{"begin", "hide_slide_button.mouseout"}, 
			{"to", "1"}, {"dur", "1s"}});
		return group;
	}
	
	/**
	 * 
	 * 1. create sub element "animate" for progress panel (rect) which 
	 * inits a new progress bar (sets duration and start time)
	 * 2. add title of slide, duration of slide and recommended resolution to 
	 * progress panel
	 * 3. add other meta infos (description, speaker, etc.) to additional info panel
	 * 
	 * @param progressPanel consists of progress bar, information about title, 
	 * duration and resolution and two buttons (1. hide annotations, 2. show 
	 * additional infos)
	 * @param additionalInfoPanel panel which shows further meta infos like speaker,
	 * description, etc.
	 * @param slide current slide object
	 * @param slideCounter is used to generate unique ID
	 * @param nrSlides number of all slides of the lecture
	 */
	public void updateProgressPanel(SvgElement progressPanel, 
			SvgElement additionalInfoPanel, Slide slide, int slideCounter, 
			int nrSlides) {
		// add progress bar animation
		long dur = slide.getDuration();
		SvgElement animate = SVGUtil.createElement("animate", progressPanel,
			new String[][] {{"id", "progress" + slideCounter}, {"to", "280"},
			{"attributeName", "width"}, {"dur", dur + "ms"}});
		// first slide
		if (slideCounter == 0) animate.addAttribute("begin", "start.click");
		else animate.addAttribute("begin", "progress" + (slideCounter - 1) + ".end");
		// freeze progress bar of last slide (slide counter starts with 0)
		if (slideCounter == (nrSlides - 1)) animate.addAttribute("fill", "freeze");
		// add slide title
		SvgElement panel = progressPanel.getParent();
		Metadata metadata = slide.getMetadata();
		String title = metadata.getTitle();
		if ((title == null) || title.trim().equals("")) {
			title = getLabel("novalue", false);
		}
		// display titel + recommended resolution + duration
		String duration = formatMillis(dur);
		title += " (" + duration + ")";
		progressPanelAddInfo(slide, panel, slideCounter, nrSlides, dur, title, 
				getLabel("title"), 10, 40, true, "12", false);
		// show label: current slide / nur of slides
		SvgElement currentSlideLabel = SVGUtil.createElement("text", panel, 
			new String[][] {{"x", "10"},{"y", "19"},{"fill", "white"},
			{"font-family", "Arial, Helvetica, sans-serif"}, {"font-size", "12"},
			{"visibility", "hidden"}});
		// first slide has slideCounter 0
		currentSlideLabel.addText(getLabel("slide") + (slideCounter + 1) + " / " + 
				nrSlides);
		// create animation subelement for currentSlideLabel to unhide element
		SvgElement showSlideLabel = SVGUtil.createElement("animate", currentSlideLabel,
			new String[][] {{"attributeName", "visibility"}, {"to", "visible"},
			{"dur", "1s"}, {"fill", "freeze"}});
		// create animation subelement for currentSlideLabel to hide element
		// (except for last slide)
		SvgElement hideSlideLabel = null;
		if (slideCounter != (nrSlides - 1)) {
			hideSlideLabel = SVGUtil.createElement("animate", currentSlideLabel,
				new String[][] {{"id", "hideSlideLabel" + slideCounter},
				{"attributeName", "visibility"}, {"to", "hidden"}, {"dur", "1s"},
				{"fill", "freeze"}});
		}
		//	first slide. 
		if (slideCounter == 0) {
			showSlideLabel.addAttribute("begin", "start.click");
			if (hideSlideLabel != null) {
				hideSlideLabel.addAttribute("begin", 
					"start.click+" + (dur - 1000) + "ms");
			}
		}
		else {
			showSlideLabel.addAttribute("begin", 
				"hideSlideLabel" + (slideCounter - 1) + ".end");
			if (hideSlideLabel != null) {
				hideSlideLabel.addAttribute("begin","hideSlideLabel" + 
					(slideCounter - 1) + ".end+" + (dur - 1000) + "ms");
			}
		}
		// add other slide information
		Person[] authors = metadata.getAuthors();
		String date = metadata.getDate();
		String description = metadata.getDescription();
		
		String site = metadata.getSite();
		Person speaker = metadata.getSpeaker();
		String snapshot = slide.getPlaceholderUrl();
		addMetaInfo(additionalInfoPanel, authors, date, title, description, site, 
			speaker, slide.getWidth() + "x" + slide.getHeight(), snapshot, 
			slideCounter, nrSlides, dur);
	}
	
	private void addMetaInfo(SvgElement panel, Person[] authors, String date, 
		String title, String desc, String site, Person speaker, String resolution, String snapshot, 
		int slideCounter, int nrSlides, long dur) {
		int lastSlideId = nrSlides - 1;
		// 'g' represents meta infos (groups all components of a set of meta infos
		// of a slide) of a slide
		// initially hidden
		SvgElement g = SVGUtil.createElement("g", panel, 
			new String[][] {{"opacity", "0"}});
		// create animation elements for making meta infos visible
		createMetaSubElements(g, slideCounter, lastSlideId, dur);
		// create 'text:wrap' elements which represents the meta infos. a text:wrap
		// element is converted into a regular svg text element at runtime
		if (!((desc == null) || (desc.trim().equals("")))) {
			title += " (" + getLabel("desc") + desc + ")";
		}
		createTextWrap(g, getLabel("title"), title);
		createTextWrap(g, getLabel("presentation"), getPerson(speaker) + ", " + 
			date + ", " + site);
		createTextWrap(g, getLabel("author"),	getPersons(authors));
		createTextWrap(g, getLabel("res"), resolution);
	}
	
	private void createTextWrap(SvgElement parent, String label, String text) {
		if ((text == null) || (text.trim().equals(""))) 
			text = getLabel("novalue", false);
		text = label + text;
		Namespace nsText = Namespace.getNamespace("text", Constants.NS_TEXTWRAP);
		SvgElement textwrap = new SvgElement("wrap", parent, nsText);	
		parent.addContent(textwrap);
		textwrap.addAttribute("style", "font-family: Arial, Helvetica, sans-serif;" + 										
  			"text-rendering: optimizeLegibility;text-align:left;color:#333399;");
		textwrap.addText(text);
	}
	
	private void createMetaSubElements(SvgElement g, int slideCounter, 
			int lastSlideId, long dur) {
		// first slide
		if (slideCounter == 0) {
			// show additional infos on document load
			SVGUtil.createElement("animate", g, 
				new String[][] {{"begin", "doc.load"}, {"to", "1"},
				{"dur", "1s"}, {"attributeName", "opacity"}, {"fill", "freeze"}});
		}
		// show info when infos of last slide are hidden 
		else {
			SVGUtil.createElement("animate", g, 
				new String[][] {{"begin", "slide" + (slideCounter-1) + "_hide.end"},
				{"attributeName", "opacity"}, {"to", "1"}, {"dur", "1s"}, 
				{"fill", "freeze"}});
		}
		// create animation elements for hiding meta infos
		// (except meta infos of last slide)
		if (!(slideCounter == lastSlideId)) {
			// first slide
			if (slideCounter == 0) {
				SVGUtil.createElement("animate", g, 
					new String[][] {{"attributeName", "opacity"}, {"to", "0"},
					{"fill", "freeze"}, {"dur", "1s"}, 
					{"begin", "start.click+" + (dur - 1000) + "ms"}});
			}
			// calculate start time for animation: dur -1s after infos of last slide 
			// are hidden
			else {
				SVGUtil.createElement("animate", g, 
					new String[][] {{"attributeName", "opacity"}, {"to", "0"},
					{"fill", "freeze"}, {"dur", "1s"}, 
					{"begin", "slide" + (slideCounter - 1)
						+ "_hide.end+" + (dur - 1000) + "ms"}});
			}
		}
	}
	
	private String getPerson(Person person) {
		if (person == null) return null;
		Person[] persons = new Person[1];
		persons[0] = person;
		return getPersons(persons);
	}
	
	
	/**
	 * @param persons
	 * @return String which consists name, company and the email adresses
	 */
	private String getPersons(Person[] persons) {
		String s;
		if (persons == null) {
			s = null;
		}
		else {
			int size = persons.length;
			int i;
			s = "";
			String name, email, company;
			String[] emails;
			for (i = 0; i < size; i++) {
				name = persons[i].getName();
				emails = persons[i].getEmail();
				email = "";
				for (int k = 0; k < emails.length; k++) {
					email += emails[k];
					if (k < (emails.length - 1)) {
						email += ", ";
					}
				}
				company = persons[i].getCompany();
				s += name + " (" + company + ", " + email + ")";
				if (i < (size - 1)) s += ", ";
			}
		}
		return s;
	}
	
	/**
	 * 
	 * add information to progress panel or additional info panel
	 * 
	 * @param slide current slide object
	 * @param panel progress panel or additional info panel
	 * @param slideCounter is used to build an unique ID
	 * @param nrSlides number of slides of lecture
	 * @param dur duration of slide 
	 * @param info the displayed information / content
	 * @param label for info
	 * @param x x-coordinate for info
	 * @param y y-coordinate for info
	 * @param title is the value of info the title string?
	 * @param fontSize font size for info
	 * @param image is the value of info a link to an image?
	 */
	private void progressPanelAddInfo(Slide slide, SvgElement panel, 
			int slideCounter, int nrSlides, long dur, String info, String label, 
			int x, int y, boolean title, String fontSize, boolean image) {
		int lastSlideId = nrSlides - 1;
		SvgElement element;
			
			// if text is longer than 75 characters substring text
			info = label + info;
			if (info.length() >= 70) {
				info = info.substring(0, 67) + "...";
			}
			element = SVGUtil.createElement("text", panel, new String[][] 
			    {{"x", x + ""}, {"y", y + ""}, {"font-size", fontSize}, 
				{"font-family", "Arial, Helvetica, sans-serif"}, {"display", "none"}});
			if (title) element.addAttribute("fill", "white");
			else element.addAttribute("fill", "#333399");
			element.addText(info);

		// define conditions for display
		// show info
		SvgElement show = SVGUtil.createElement("animate", element, new String[][] {
			{"id", "show" + slideCounter}, {"attributeName", "display"}, 
			{"to", "inline"}, {"fill", "freeze"}, {"dur", "1s"}});
		// first slide. Show info when start button is clicked
		if (slideCounter == 0) {
			// show additional infos on doc load
			if (!title) show.addAttribute("begin", "doc.load");
			// show title 
			else show.addAttribute("begin", "start.click");
		}
		// show info when infos of last slides are hidden 
		else {
			show.addAttribute("begin", "slide" + (slideCounter - 1) 
				+ "_hide.end");
		}
		// hide info (except information of last slide)
		if (!(slideCounter == lastSlideId)) {
			SvgElement hide = SVGUtil.createElement("animate", element, 
				new String[][] {{"attributeName", "display"}, {"to", "none"},
				{"fill", "freeze"}, {"dur", "1s"}});
			// first slide. 
			if (slideCounter == 0) {
				// dur - 1000 -> because animation lasts 1s
				hide.addAttribute("begin", "start.click+" + (dur - 1000) + "ms");
			}
			// calculate start time for animation: dur -1s after infos of last slide 
			// are hidden
			else {
				hide.addAttribute("begin", "slide" + (slideCounter - 1)
							+ "_hide.end+" + (dur - 1000) + "ms");
			}
		}
	}
	
	
	/**
	 * 
	 * create additional info panel
	 * 
	 * @param progressPanel is root element of additional info panel
	 * @return reference to additional info panel
	 */
	public SvgElement initAdditionalInfoPanel(SvgElement progressPanel, 
		Slide[] slides) {
		int nrSlides = slides.length;
		// all components of panel are grouped. by default invisible. On mouseover
		// (additional info button) show panel
		SvgElement additionalInfoPanel = SVGUtil.createElement("g", progressPanel,
			new String[][] {{"display", "block"}, {"id", "additionalInfo"}});
		//  hide it when user clicks on start button
		SVGUtil.createElement("set", additionalInfoPanel, new String[][] 
		    {{"attributeName", "display"}, {"begin", "start.click"},{"to", "none"}});
		// make panel visible on mouseover
		SVGUtil.createElement("animate", additionalInfoPanel,
			new String[][] {{"attributeName", "display"}, {"dur", "1s"},
			{"begin", "additional_info_button1.click"}, {"to", "block"}, 
			{"fill", "freeze"}});
		// hide panel on mouseout
		SVGUtil.createElement("animate", additionalInfoPanel, new String[][] 
		    {{"attributeName", "display"}, {"dur", "1s"}, {"to", "none"}, 
			{"begin", "additional_info_button2.click"}, {"fill", "freeze"}});
		// create clip path for drop shaddow
		SvgElement defs = SVGUtil.createElement("defs", additionalInfoPanel);
		SvgElement clipPath = SVGUtil.createElement("clipPath", defs, 
			new String[][] {{"id", "clip_progresspanel_info"}});
		// y variable
		SvgElement clip_bottom = SVGUtil.createElement("rect", clipPath, 
			new String[][] {{"x", "10"}, {"y", "355"}, 
			{"width", "420"}, {"height", "5"}});
		// height variable
		SvgElement clip_right = SVGUtil.createElement("rect", clipPath, 
			new String[][] {{"x", "425"}, {"y", "60"}, 
			{"width", "5"}, {"height", "295"}});
		// create panel (rectangle)
		// height variable
		SvgElement infoPanel = SVGUtil.createElement("rect", additionalInfoPanel, 
			new String[][] 
		    {{"x", "5"}, {"y", "55"}, {"width", "420"}, 
			{"height", "300"}, {"fill", "#99CCFF"}, {"stroke", "grey"}, 
			{"stroke-width", "1"}});
		// add drop shaddow to panel
		// height variable
		SvgElement dropShaddow = SVGUtil.createElement("rect", additionalInfoPanel,
			new String[][] {{"x", "10"}, 
			{"y", "60"}, {"width", "420"}, {"height", "300"}, {"fill", "black"}, 
			{"opacity", "0.4"}, {"clip-path", "url(#clip_progresspanel_info)"}});
		// one image element wich represents the snapshot of a slide. Attributes
		// will be set for every slide individually 
		// x, y and href will be calculated by an external script at runtime
		// height is fix, width will be calculated for every slide
		SvgElement image = SVGUtil.createElement("image", additionalInfoPanel, 
			new String[][] {{"height", "155"}});
		Namespace xlink = Namespace.getNamespace("xlink", Constants.NS_XLINK);
		// href will be set for every slide
		image.addAttribute("href", "", xlink);
		// create set elements to alter attributes of the panel, its drop shaddow
		// and snapshot for every slide
		int i;
		Slide slide;
		for (i = 0; i < nrSlides; i++) {
			slide = slides[i];
			// first slide
			if (i == 0) {
				// attribute 'to' will be set by external script which detects
				// dynamic 'y'-value
				SVGUtil.createElement("set", clip_bottom, 
					new String[][] {{"begin", "doc.load"}, {"attributeName", "y"},
					{"id", "clip_bottom" + i}});
				// attribute 'height' will be set by external script which detects
				// dynamic 'height'-value
				SVGUtil.createElement("set", clip_right, 
					new String[][] {{"id", "clip_right" + i}, 	{"begin", "doc.load"}, 
					{"attributeName", "height"}});
				SVGUtil.createElement("set", infoPanel, 
					new String[][] {{"id", "progresspanel" + i}, 
					{"begin", "doc.load"}, {"attributeName", "height"}});
				SVGUtil.createElement("set", dropShaddow, 
					new String[][] {{"id", "dropshaddow" + i}, 
						{"begin", "doc.load"}, {"attributeName", "height"}});
				addSetImage(image, slide, "doc.load", i);
			}
			else {
				// attribute 'to' will be set by external script which detects
				// dynamic 'y'-value
				SVGUtil.createElement("set", clip_bottom, 
					new String[][] {{"begin", "slide" + (i - 1) + "_hide.end"}, 
					{"attributeName", "y"}, {"id", "clip_bottom" + i}});
				// attribute 'height' will be set by external script which detects
				// dynamic 'height'-value
				SVGUtil.createElement("set", clip_right, 
					new String[][] {{"id", "clip_right" + i}, 	
					{"begin", "slide" + (i - 1) + "_hide.end"},
					{"attributeName", "height"}});
				SVGUtil.createElement("set", infoPanel, 
					new String[][] {{"id", "progresspanel" + i}, 
					{"begin", "slide" + (i - 1) + "_hide.end"}, 
					{"attributeName", "height"}});
				SVGUtil.createElement("set", dropShaddow, 
					new String[][] {{"id", "dropshaddow" + i}, 
					{"begin", "slide" + (i - 1) + "_hide.end"}, 
					{"attributeName", "height"}});
				addSetImage(image, slide, "slide" + (i - 1) + "_hide.end", i);
			}
		}
		return additionalInfoPanel;
	}
	
	private void addSetImage(SvgElement image, Slide slide, String begin, 
		int index) {
		// add set element to update href attribute of image
		SVGUtil.createElement("set", image, 
			new String[][] {{"attributeName", "href"}, 
			{"to", slide.getPlaceholderUrl()}, {"begin", begin}});
		// calculate width, height is fix (155px)
		double canvasHeight = slide.getHeight();
		double scaleFactor = canvasHeight / 155;
		double canvasWidth = slide.getWidth();
		int width = (int) (canvasWidth / scaleFactor); 
		SVGUtil.createElement("set", image, new String[][] {
			{"attributeName", "width"}, {"to", width + ""}, {"begin", begin}});
		// add 2 set elements for 'x' and 'y'. These values will be calculated at
		// runtime. 'to' attribute will be added by an external script at runtime
		// add id to accomplish this
		SVGUtil.createElement("set", image, 
			new String[][] {{"id", "snapshot_x" + index}, {"attributeName", "x"}, 
			{"begin", begin}});
		SVGUtil.createElement("set", image, 
			new String[][] {{"id", "snapshot_y" + index}, {"attributeName", "y"}, 
			{"begin", begin}});
	}
	
	/**
	 * 
	 * create progress panel with progess bar and buttons
	 * 
	 * @param root progress panel is attached to root 
	 * @param nrSlides number of slides of lecture
	 * @param dur duration of current slide
	 * @return reference to progress panel
	 */
	public SvgElement initProgressPanel(SvgElement root, int nrSlides, long dur) {
		// all components of progress panel are grouped
		SvgElement panel = SVGUtil.createElement("g", root, 
			new String[][] {{"id", "progress"}, {"opacity", "1"}});
		// by default opacity of panel is 100%. when cursor position is over panel
		// or one of its components (progress bar, buttons, etc.) set opacity to 
		// 100%. On panel mouseout animate opacity to 20%.
		initMouseEvents(panel);
		// create timer
		addTimer(panel);
		// create info panel
		SvgElement infoPanel = SVGUtil.createElement("g", panel, 
			new String[][] {{"id", "info-panel"}});
		// create rectangle for progress panel
		SVGUtil.createElement("rect", infoPanel, 
			new String[][] {{"id", "progress_panel"}, {"x", "5"}, {"y", "5"}, 
			{"width", "420"}, {"height", "40"}, 
			{"fill", "#006699"}, {"stroke", "grey"}, {"stroke-width", "1"}});
		// add labels for progress bar
		addLabelProgressBar(infoPanel);
		SvgElement progressBar = addProgressBar(infoPanel);
		// add buttons to show meta infos, hide annotations of current slide and 
		// start the animations
		addButtons(root, infoPanel, dur);
		addLectureTitle(infoPanel, nrSlides, dur);
		return progressBar;
	}
	
	private void addTimer(SvgElement panel) {
		SvgElement g = SVGUtil.createElement("g", panel, 
			new String[][] {{"id", "timer_g"}, {"onclick", "switchTimer()"}});
		SVGUtil.createElement("rect", g, new String[][] {{"x", "440"}, 
			{"y", "5"}, {"width", "180"}, {"height", "40"}, 
			{"fill", "#006699"}, {"stroke", "grey"}, {"stroke-width", "1"}});	
		SVGUtil.createElement("text", g, 
			new String[][] {{"id", "timer"},{"x", "470"}, {"y", "37"}, 
			{"fill", "white"}, {"font-family", "Arial, Helvetica, sans-serif"}, 
			{"font-size", "36"}, {"opacity", "1"}}, "0:00:00");
		SVGUtil.createElement("text", g, 
				new String[][] {{"id", "timer2"},{"x", "465"}, {"y", "37"}, 
				{"fill", "white"}, {"font-family", "Arial, Helvetica, sans-serif"}, 
				{"font-size", "36"}, {"opacity", "0"}}, "-0:00:00");
		// drop shaddow
		SvgElement defs = SVGUtil.createElement("defs", g);
		SvgElement clipPath = SVGUtil.createElement("clipPath", defs, 
			new String[][] {{"id", "clip_timer"}});
		SVGUtil.createElement("rect", clipPath, new String[][] {{"x", "445"},
			{"y", "45"}, {"width", "180"}, {"height", "5"}});
		SVGUtil.createElement("rect", clipPath, new String[][] {{"x", "620"},
				{"y", "10"}, {"width", "5"}, {"height", "40"}});
		SVGUtil.createElement("rect", g, new String[][] {{"x", "445"},
			{"y", "10"}, {"height", "40"}, {"width", "185"}, {"fill", "black"},
			{"opacity", "0.4"}, {"clip-path", "url(#clip_timer)"}});
	}
	
	
	private void addLabelProgressBar(SvgElement panel) {
		SvgElement progressText = SVGUtil.createElement("text", panel, 
			new String[][] {{"id", "progress_text"}, {"x", "10"}, {"y", "19"},
			{"fill", "white"}, {"font-family", "Arial, Helvetica, sans-serif"},
			{"font-size", "12"}, {"visibility", "visible"}});
		progressText.addText(getLabel("progress"));
		// hide label when start button is clicked
		SVGUtil.createElement("set", progressText, new String[][] 
		    {{"attributeName", "visibility"}, {"begin", "start.click"}, 
			{"to", "hidden"}});
		SvgElement label1 = SVGUtil.createElement("text", panel, 
			new String[][] {{"id", "label1"}, {"x", "80"}, {"y", "19"}, 
			{"fill", "white"}, {"font-family", "Arial, Helvetica, sans-serif"},
			{"font-size", "12"}});
		label1.addText("0%");
		SvgElement label2 = SVGUtil.createElement("text", panel, 
			new String[][] {{"id", "label2"}, {"x", "382"}, {"y", "19"}, 
			{"fill", "white"}, {"font-family", "Arial, Helvetica, sans-serif"},
			{"font-size", "12"}});
		label2.addText("100%");
	}
	
	private SvgElement addProgressBar(SvgElement panel) {
		SVGUtil.createElement("rect", panel, 
			new String[][] {{"id", "progress_border"}, {"x", "100"}, {"y", "10"},
			{"fill", "none"}, {"stroke", "grey"}, {"stroke-width", "1"},
			{"height", "10"}, {"width", "280"}});
		// drop shaddow of panel
		SvgElement defs = SVGUtil.createElement("defs", panel);
		SvgElement clipPath = SVGUtil.createElement("clipPath", defs,
			new String[][] {{"id", "clip_progresspanel"}});
		SVGUtil.createElement("rect", clipPath, new String[][] {{"x", "10"}, 
			{"y", "45"}, {"width", "415"}, {"height", "5"}});
		SVGUtil.createElement("rect", clipPath, new String[][] {{"x", "425"},
			{"y", "10"}, {"width", "5"}, {"height", "40"}});
		SVGUtil.createElement("rect", panel,
			new String[][] {{"x", "10"}, {"y", "10"}, {"height", "40"}, 
			{"width", "420"}, {"fill", "black"}, {"opacity", "0.4"},
			{"clip-path", "url(#clip_progresspanel)"}});
		// create rectangle (progress bar) 
		SvgElement progressBar = SVGUtil.createElement("rect", panel,
			new String[][] {{"id", "progress_bar"}, {"x", "100"}, {"y", "10"},
			{"fill", "white"}, {"height", "10"}, {"width", "0"}});
		return progressBar;
	}
	
	private void addLectureTitle(SvgElement root, int nrSlides, long dur) {
		// show lecture info (nr slides, duration)
		SvgElement info = SVGUtil.createElement("text", root, 
			new String[][] {{"x", "95"}, {"y", "37"},
			{"style", "text-anchor: start; font-size: 14px;"  +
				"font-family: Helvetica; font-weight: bold; fill:white;"}});
		info.addText(getLabel("nrSlides") + nrSlides + ", " + getLabel("dur") + 
			formatMillis(dur));
		// hide info text when start button was clicked
		SVGUtil.createElement("set", info, new String[][] {{"begin", "start.click"},
			{"attributeName", "display"}, {"to", "none"}});
	}
	
	private void initMouseEvents(SvgElement panel) {
		infoPanelHelper_set(panel, "mouseover", "1");
		infoPanelHelper_set(panel, "additional_info_button1.mouseover", "1");
		infoPanelHelper_set(panel, "additional_info_button1.mouseout", "1");
		infoPanelHelper_set(panel, "additional_info_button1.click", "1");
		infoPanelHelper_set(panel, "additional_info_button2.click", "1");
		infoPanelHelper_set(panel, "timer_g.click", "1");
	}
	
	private void addLinearGradient(SvgElement root) {
		SvgElement gradient = SVGUtil.createElement("linearGradient", root,
			new String[][] {{"id", "verlaufweiss"}, {"x1", "0%"}, {"y1", "0%"},
			{"x2", "0%"}, {"y2", "100%"}, {"spreadMethod", "pad"}});
		SVGUtil.createElement("stop", gradient, 
			new String[][] {{"offset", "0%"}, {"stop-color", "white"}});
		SVGUtil.createElement("stop", gradient,
			new String[][] {{"offset", "50%"}, {"stop-color", "grey"}});
		SVGUtil.createElement("stop", gradient, 
			new String[][] {{"offset", "100%"}, {"stop-color", "white"}});
	}
	
	private void addButtons(SvgElement root, SvgElement panel, long dur) {
		addAdditionalInfoButton(panel);
		addHideAnnotationsButton(panel, root);
		addStartButton(root, dur);
	}
	
	private void addAdditionalInfoButton(SvgElement panel) {
		// all components of the button are grouped
		SvgElement additionalInfoButton = SVGUtil.createElement("g", panel,
			new String[][] {{"id", "additional_info_button1"}, 
			{"onclick", "infoButtonAlterID(evt)"}});
		// background shape
		SVGUtil.createElement("rect", additionalInfoButton,
			new String[][] {{"fill", "red"}, {"stroke", "silver"}, 
			{"stroke-width", "1px"}, {"x", "385"}, {"y", "26"}, {"width", "16"}, 
			{"height", "16"}});
		// triangle symbol for the button
		SVGUtil.createElement("polygon", additionalInfoButton,
			new String[][] {{"fill", "white"}, {"points", "389,30 397,30 393,38"}});
	}
	
	private void addHideAnnotationsButton(SvgElement panel, SvgElement root) {
		SVGUtil.createElement("rect", panel, 
			new String[][] {{"id", "hide_slide_button"}, {"fill", "url(#gradientID)"},
			{"x", "406"}, {"y", "26"}, {"width", "16"}, {"height", "16"}, 
			{"stroke", "silver"}});
		SvgElement defsGradient = SVGUtil.createElement("defs", panel);
		SvgElement gradient = SVGUtil.createElement("linearGradient", defsGradient,
			new String[][] {{"id", "gradientID"}});
		SVGUtil.createElement("stop", gradient, 
			new String[][] {{"offset", "0%"}, {"stop-color", "#006699"}});
		SVGUtil.createElement("stop", gradient,
			new String[][] {{"offset", "100%"}, {"stop-color", "white"}});
		// add to root element (and not group) to make gradient accessible for other 
		// elements
		addLinearGradient(root);
	}

	private void addStartButton(SvgElement root, long dur) {
		// on click invoke JS function which adds elements at runtime
		SvgElement group = SVGUtil.createElement("g", root, 
			new String[][] {{"id", "start"}, 
			{"onclick", "addElementsToProgressbar(evt); setupTimer(" + dur +");" +
			"updateContextMenu();"}});
		SVGUtil.createElement("set", group, new String[][] {{"to", "none"},
			{"begin", "click"}, {"attributeName", "display"}});
		// button
		SVGUtil.createElement("rect", group, new String[][] {{"x", "10"},
			{"y", "25"}, {"width", "75"}, {"height", "16"}, {"rx", "8"}, {"ry", "8"},
			{"fill", "url(#verlaufweiss)"}, {"stroke", "grey"}, 
			{"stroke-width", "1"}});
		// label
		SvgElement label = SVGUtil.createElement("text", group, 
			new String[][] {{"x", "23"}, {"y", "37"}, 
			{"style", "text-anchor: start; font-size: 14px;"  +
				"font-family: Helvetica; font-weight: bold; fill:black;"}});
		label.addText(getLabel("start", false));
	}
	
	private void infoPanelHelper_set(SvgElement root, String begin, String to) {
		SVGUtil.createElement("set", root, new String[][] 
		    {{"attributeName", "opacity"}, {"begin", begin}, {"to", to}});
	}
	
	/**
	 * 
	 * Transform milliseconds into minute : seconds
	 * 
	 * @param ms milliseconds
	 * @return transformed format as string
	 * 
	 */
	public String formatMillis(long ms) {
		// minute:seconds
		long sec =  ms / 1000;
		String seconds = (sec % 60) + "";
		if (seconds.length() == 1) seconds = "0" + seconds;
		String minutes = ((int) sec / 60) + "";
		return minutes + ":" + seconds;
	}
	
	public String getLabel(String key, boolean flag) {
		XPath x;
		List results;
		try {
			x = XPath.newInstance("//label[@name='" + key + "']");
			results = x.selectNodes(labels);
		} catch (JDOMException e) {
			return "?";
		}
		
		if (results.size() == 0) {
			System.err.println("Error: illegal language id '" + lang + "'");
			System.exit(0);
		}
		Element e = (Element) results.get(0);
		String label = e.getChild(lang).getText();
		if (flag) label += ":";
		return label + " ";
	}
	
	public String getLabel(String key) {
		return getLabel(key, true);
	}
}
