package eLecture;

import java.util.ArrayList;
import java.util.List;


import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import eLecture.annotation.Annotation;
import eLecture.annotation.Freehand;
import eLecture.annotation.Laserpointer;
import eLecture.annotation.Rectangle;
import eLecture.annotation.Textbox;

/**
 * @author seb
 *
 * Wrap lecture information of XML input file. Provide access methods for lecture 
 * metadata, topics and slides. InputWrapper uses XML DOM tree and constructs 
 * concrete objects which wrapp the information provided by the input file.
 * 
 */
public class InputWrapper {
	
	/**
	 * the input xml file as DOM tree
	 */
	private Document doc;
	/**
	 * this is the namespace prefix of the xml input file
	 */
	private String namespacePrefix;
	/**
	 * the meta data of the lecture
	 */
	private Metadata metadata;
	/**
	 * the list of topics, at least one element
	 */
	private Topic[] topics = null;
	
	public InputWrapper(Document doc) {
		this.doc = doc;
		namespacePrefix = doc.getRootElement().getNamespacePrefix();
		// retrieve metadata of lecture and create Metadata object
		Element lectureMeta = null;
		Element topicMeta = null;
		Element slideMeta = null;
		List results = getResults("/" + namespacePrefix + ":lecture/metadata");
		if (results.size() != 0) {
			lectureMeta = (Element) results.get(0);
		}
		results = getResults("//topic/metadata"); 
		if (results.size() != 0) {
			topicMeta = (Element) results.get(0);
		}
		results = getResults("//slide/metadata"); 
		if (results.size() != 0) {
			slideMeta = (Element) results.get(0);
		}
		metadata = new Metadata(lectureMeta, topicMeta, slideMeta);
		// retrieve and proceed topics
		proceedTopics();
	}
	
	/**
	 * 
	 * calculate duration of complete lecture by retrieving all slides of all topics.
	 * 
	 * @return duration in millis
	 */
	public long getDuration() {
		long dur = 0;
		Slide[] currentSlides;
		Slide current;
		int k;
		int nrTopics = topics.length;
		int i;
		for (i = 0; i < nrTopics; i++) {
			currentSlides = topics[i].getSlides();
			for (k = 0; k < currentSlides.length; k++) {
				current = currentSlides[k];
				dur += current.getDuration();
			}
		}
		return dur;
	}
	
	/**
	 * 
	 * retrieve all topic elements of input file. For every topic create a 
	 * Topic object.
	 * 
	 */
	private void proceedTopics() {
		List results = getResults("//topic");
		// iterate all topics
		int numberOfTopics = results.size();
		this.topics = new Topic[numberOfTopics];
		int i;
		Topic topic;
		Slide[] slides;
		Element topicElement, topicMetadata, lectureMetadata;
		// metadata of current topic
		Metadata metadata;
		for (i = 0; i < numberOfTopics; i++) {
			topicElement = (Element) results.get(i);
			slides = retrieveSlides(topicElement);
			// retrieve metadata of current topic
			topicMetadata = topicElement.getChild("metadata");
			// get lecture metadata
			List r = getResults("/" + namespacePrefix + ":lecture/metadata"); 
			if (r.size() != 0) {
				lectureMetadata = (Element) r.get(0);
			}
			else lectureMetadata = null;
			// do not pass slide metadata. pass topicMetadata as first argument and
			// lectureMetadata as 2nd
			metadata = new Metadata(topicMetadata, lectureMetadata, null);
			topic = new Topic(slides, metadata);
			// add current topic to list of all topics
			this.topics[i] = topic;
		}
	}
	
	/**
	 * 
	 * retrieve all slides of a given topic element.
	 * 
	 * @param topic XML element 
	 * @return slides of given topic
	 */
	private Slide[] retrieveSlides(Element topic) {
		Slide[] slides = null;
		Slide slide;
		List results = topic.getChildren("slide");
		int numberOfSlides = results.size();
		slides = new Slide[numberOfSlides];
		int i;
		for (i = 0; i < numberOfSlides; i++) {
			// get Slide object and add it to list
			slide = initSlide((Element) results.get(i));
			slides[i] = slide;
		}
		return slides;
	}
	
	/**
	 * 
	 * create Slide object
	 * 
	 * @param slideElement XML element
	 * @return Slide object
	 */
	private Slide initSlide(Element slideElement) {
		Slide slide = null;
		// retrieve meta data
		Element slideMeta = slideElement.getChild("metadata");
		Element lectureMeta, topicMeta;
		List r = getResults("/" + namespacePrefix + ":lecture/metadata"); 
		if (r.size() != 0) {
			lectureMeta = (Element) r.get(0);
		}
		else lectureMeta = null;
		r = getResults("//topic/metadata"); 
		if (r.size() != 0) {
			topicMeta = (Element) r.get(0);
		}
		else topicMeta = null;
		Metadata metadata = new Metadata(slideMeta, topicMeta, lectureMeta);
		// get width of slide
		int width = Integer.parseInt(
				slideElement.getAttributeValue("canvaswidth").trim());
		// height of slide
		int height = Integer.parseInt(
				slideElement.getAttributeValue("canvasheight").trim());
		// duration of slide
		long duration = Long.parseLong(
				slideElement.getAttributeValue("duration").trim());
		// retrieve media information
		Element mediaElement = slideElement.getChild("media");
		// url to background image
		String pic = getAttributeValue("picture", "filename", mediaElement);
		// url to audio file
		String audio = getAttributeValue("audio", "filename", mediaElement);
		// url to snapshot
		String snapshot = getAttributeValue("snapshot", "filename", mediaElement);
		String url = getAttributeValue("link", "url", mediaElement);
		// retrieve annotations
		Element annotationElement = slideElement.getChild("annotation");
		Annotation[] annotations;
		// slide contains no annotations
		if (annotationElement == null) annotations = null;
		else annotations = retrieveAnnotations(annotationElement);
		// construct Slide object
		slide = new Slide(pic, snapshot, audio, url, annotations, duration, width,
				height, metadata);
		return slide;
	}
	
	/**
	 * 
	 * return attribute value 
	 * 
	 * @param elementName name of the element
	 * @param attr attribute name
	 * @param root is father element
	 * @return attribute value
	 */
	private String getAttributeValue(String elementName, String attr, Element root) {
		String value = null;
		Element current = root.getChild(elementName);
		if (current != null) {
			value = current.getAttributeValue(attr);
		}
		return value;
	}
	
	/**
	 * 
	 * retrieve annotations of slide element. annotations are one of text, freehand,
	 * laserpointer or rectangle. Annotation is the abstract superclass of the 
	 * concrete sub classes Freehand, Textbox, Rectangle or Laserpointer.
	 * 
	 * @param root slide element
	 * @return list of all annotations
	 */
	private Annotation[] retrieveAnnotations(Element root) {
		// children are annotation elements
		List results = root.getChildren();
		// number of annotations
		int annotationSize = results.size();
		// init list of annotations
		Annotation[] annotations = new Annotation[annotationSize];
		Element currentAnnotation;
		// iterate all annotations of a slide
		int i;
		Annotation annotation;
		for (i = 0; i < annotationSize; i++) {
			currentAnnotation = (Element) results.get(i);
			// laserpointer, freehand, rectangle or text
			String name = currentAnnotation.getName();	
			// create concrete Annotation object
			// textbox
			if (name.equals("text")) {
				annotation = addTextbox(currentAnnotation);
			}
			// freehand
			else if (name.equals("freehand")) {
				annotation = addFreehand(currentAnnotation);
			}
			// textmarker
			else if (name.equals("rectangle")) {
				annotation = addRectangle(currentAnnotation);
			}
			// laserpointer
			else {
				annotation = addLaserpointer(currentAnnotation);
			}
			annotations[i] = annotation;
		}
		return annotations;
	}
	
	/**
	 * 
	 * retrieve location elements of textmarker, freehand or laserpointer. return
	 * list of all locations.
	 * 
	 * @param annotation textmarker, freehand or laserpointer
	 * @param optimize if true then use only every fifth location
	 * @return list of all annotations
	 */
	private Location[] retrieveLocations(Element annotation, boolean optimize) {
		// child elements of 
		List locations = annotation.getChildren();
		int k;
		// number of locations of given annotation
		int length = locations.size();
		// temp location list
		ArrayList locationList = new ArrayList();
		Location location, prevLocation;
		Element currentLocation;
		String name = annotation.getName();
		// helper variable for optimization
		int i = 0;
		for (k = 0; k < length; k++) {
			int x = 0;
			int y = 0;
			long offset = 0;
			// offset from previous location
			long prevOffset;
			int offsetx = 0;
			int offsety = 0;
			currentLocation = (Element) locations.get(k);
			if (name.equals("laserpointer") | name.equals("freehand")) {
				x = Integer.parseInt(currentLocation.
						getAttributeValue("x").trim());
				y = Integer.parseInt(currentLocation.
						getAttributeValue("y").trim());
			}
			else {
				offsetx = Integer.parseInt(currentLocation.
						getAttributeValue("offsetx").trim());
				offsety = Integer.parseInt(currentLocation.
						getAttributeValue("offsety").trim());
			}
			offset = Long.parseLong(currentLocation.
					getAttributeValue("offset").trim());
			
			// discard spans / locations with duration 0 (i.e. previous offset =
			// current offset) only for textmarker.
			// not for laserpointer and freehand because there can be curves which 
			// have to be drawn in very small intervals (resolution problem of
			// annotation tool). In the xml input file this effect appears in 
			// several location-tags which have the same offset
			// get previous location / span
			boolean ok = true;
			if (name.equals("rectangle")) {
				if (k > 0) {
					int lastIndex = locationList.size() - 1;
					prevLocation = (Location) locationList.get(lastIndex);
					prevOffset = prevLocation.getOffset();
					if (prevOffset == offset) {
						ok = false;
					} 
				}
			}
			if (ok) {
				// if flag = true than use only every fifth location 
				// (discard 4 of 5)
				if (optimize) i++;
				// use location
				else i = 1;
				if (i == 1) {
					location = new Location(offset, x, y, offsetx, offsety);
					locationList.add(location);
				}
				if (i == 5) i = 0;
			}
		}
		// return array and not an array list
		return computeLocationArray(locationList);
	}
	
	
	/**
	 * 
	 * create an array of locations 
	 * 
	 * @param list ArrayList
	 * @return Array of locations
	 */
	private Location[] computeLocationArray(ArrayList list) {
		int size = list.size();
		Location[] array = new Location[size];
		Location current;
		int i;
		for (i = 0; i < size; i++) {
			current = (Location) list.get(i);
			array[i] = current;
		}
		return array;
	}
	
	/**
	 * 
	 * retun Annotation-object from XML element 
	 * 
	 * @param e XML element
	 * @param type of annotation: freehand, laserpointer or rectangle
	 * @return concrete Annotation-object
	 */
	private Annotation getAnnotation(Element e, String type) {
		// get and format transparency
		String transparency = transformFormat(e.
				getAttributeValue("transparency"));
		// get point of start of annotation in millis
		long timestamp = Long.parseLong(e.getAttributeValue("timestamp").trim());
		// get and validate color (whether it is a valid web color description)
		String color = validateColor(e.getAttributeValue("color").trim());
		// locations of the annotation
		Location[] locations = null;
		// textboxes have no location sub elements
		// create concrete Annotation object
		Annotation annotation = null;
		if (type.equals("freehand")) {
			locations = retrieveLocations(e, false);
			String diameter = e.getAttributeValue("diameter").trim();
			annotation = new Freehand(timestamp, color, diameter, 
				transparency, locations); 
		}
		else if (type.equals("laserpointer")) {
			locations = retrieveLocations(e, false);
			String diameter = e.getAttributeValue("diameter");
			annotation = new Laserpointer(timestamp, color, diameter, 
				transparency, locations); 		
		}
		else if (type.equals("rectangle")) {
			locations = retrieveLocations(e, true);
			long duration = Long.parseLong(e.	getAttributeValue("duration").trim());
			int x = Integer.parseInt(e.getAttributeValue("x").trim());
			int y = Integer.parseInt(e.getAttributeValue("y").trim());
			String sOffsetx = e.getAttributeValue("offsetx").trim();
			int offsetx = 0;
			if (sOffsetx != null) {
				Integer.parseInt(sOffsetx);
			}
			int offsety = 0;
			String sOffsety = e.getAttributeValue("offsety").trim();
			if (sOffsety != null) {
				Integer.parseInt(sOffsety);
			}
			annotation = new Rectangle(timestamp, color, transparency, 	locations, 
				duration, x, y, offsetx, offsety);	
		}
		else if (type.equals("text")) {
			long duration = Long.parseLong(e.getAttributeValue("duration").trim());
			String fontFamily = e.getAttributeValue("font");
			String fontStyle = e.getAttributeValue("fontstyle");
			double fontSize = Double.parseDouble(e.
					getAttributeValue("fontsize").trim());
			String fillType = e.getAttributeValue("type");
			String textColor = e.getAttributeValue("textcolor").trim();
			int x = Integer.parseInt(e.getAttributeValue("x").trim());
			int y = Integer.parseInt(e.getAttributeValue("y").trim());
			int connectX = Integer.parseInt(e.
					getAttributeValue("connectx").trim());
			int connectY = Integer.parseInt(e.getAttributeValue("connecty").trim());
			int width = Integer.parseInt(e.getAttributeValue("width").trim());
			int height = Integer.parseInt(e.getAttributeValue("height").trim());
			// get text lines
			List children = e.getChildren();
			int size = children.size();
			String[] lines = new String[size];
			String line;
			int i;
			for (i = 0; i < size; i++) {
				line = ((Element) children.get(i)).getText();
				lines[i] = line;
			}
			annotation = new Textbox(timestamp, duration, fontFamily, fontStyle,
					fontSize, color, textColor, x, y, connectX, connectY, width,
					height, transparency, lines, fillType);
		}
		return annotation;
	}
	
	/*
	 * there are 3 possibilities to describe web colors:
	 * 1. color names
	 * 2. hex 6
	 * 3. hex 3
	 * 
	 * if input don't fit to 1-3 then return default value "black"
	 */
	private String validateColor(String value) {
		// default value;
		String color = "black";
		// HTML 4.x specification defines sixteen color names
		// everything else is a proprietary color name
		String[] colorNames = {"black", "silver", "gray", "white", 
				"maroon", "red", "purple", "fuchsia", "green", "lime", "olive", 
				"yellow", "navy", "blue", "teal", "aqua"};
		// hex value
		if (value.startsWith("#")) {
			// hex 6: #xxxxxx
			int i;
			if (value.length() == 7) {
				i = 6;
			}
			// hex 3: #xxx
			else if (value.length() == 4) {
				i = 3;
			}
			// illegal value
			else return color;
			// check if every character has one of the values: 0-9, A, B, C, D, E, F
			String[] hex = {"0", "1", "2", "3", "4", "5", "6", "7", "8",
					"9", "A", "B", "C", "D", "E", "F", "a", "b", "c", "d", "e", 
					"f"};
			// start with 2nd character (first is #)
			String c;
			for (int k = 1; k <= i; k++) {
				c = value.substring(k, k + 1);
				boolean ok = false;
				for (int l = 0; l < hex.length; l++) {
					if (c.equals(hex[l])) {
						ok = true;
						break;
					}
				}
				if (!ok) return color;
			}
			// valid value
			color = value;
		}
		// color name
		else {
			boolean ok = false;
			for (int i = 0; i < colorNames.length; i++) {
				if (value.equals(colorNames[i])) ok = true;
			}
			if (!ok) return color;
			// valid value
			else color = value;
		}
		return color;
	}
	
	/**
	 * 
	 * out of XML element construct a Freehand-object
	 * 
	 * @param freehandElement XML element
	 * @return Freehand-object
	 */
	private Freehand addFreehand(Element freehandElement) {
		Freehand freehand = (Freehand) getAnnotation(freehandElement, "freehand");
		return freehand;
	}
	
	/**
	 * 
	 * out of XML element construct a Laserpointer-object
	 * 
	 * @param laserpointerElement XML element
	 * @return Laserpointer-object
	 */
	private Laserpointer addLaserpointer(Element laserpointerElement) {
		Laserpointer laserpointer = (Laserpointer) getAnnotation(laserpointerElement,
				"laserpointer");
		return laserpointer;
	}
	
	/**
	 * 
	 * out of XML element construct a Rectangle-object
	 * 
	 * @param rectangleElement XML element
	 * @return Rectangle-object
	 */
	private Rectangle addRectangle(Element rectangleElement) {
		Rectangle rectangle = (Rectangle) getAnnotation(rectangleElement, 
				"rectangle");
		return rectangle;
	}
	
	/**
	 * 
	 * out of XML element construct a Textbox-object
	 * 
	 * @param testElement XML element
	 * @return Textbox-object
	 */
	private Textbox addTextbox(Element textElement) {
		Textbox textbox = (Textbox) getAnnotation(textElement, "text");		
		return textbox;
	}
	
	/**
	 * 
	 * Transform format of transparency: e.x. 80% into 0.8
	 * 
	 * @param transparency input String (value between 0 and 100 with %-sign)
	 * @return value between 0 and 1
	 */
	private String transformFormat(String transparency) {
		if ((transparency == null) || (transparency.trim().equals(""))) {
			return "0";
		}
		int stringSize = transparency.length();
		if (stringSize > 0) {
			// remove % (last character)
			transparency = transparency.substring(0, stringSize - 1);
			try {
				double newValue;
				int temp = Integer.parseInt(transparency);
				if ((temp < 0) | (temp > 100)) return "0";
				
				newValue = (double) temp / 100;
				transparency = newValue + "";
			} catch (NumberFormatException e) {
				return "0";
			}
			
		}
		else transparency = "0";
		return transparency;
	}
	
	/**
	 * 
	 * for a given xpath expression return result list
	 * 
	 * @param expr xpath expression
	 * @return result list of query
	 */
	private List getResults(String expr) {
		List results = null;
		try {
			XPath x = XPath.newInstance(expr);
			results = x.selectNodes(doc);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		return results;
	}

	
	/**
	 * @return list of topics of the lecture
	 */
	public Topic[] getTopics() {
		return topics;
	}

	/**
	 * @return metadata of the lecture
	 */
	public Metadata getMetadata() {
		return metadata;
	}
}
